#Include <Misc.au3>
#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>

$hotkey = IniRead("settings.ini", "hotkeys", "hotkey", "{SPACE}")

$tryVlasisku = IniRead("settings.ini", "settings", "tryvlasisku", 0)

$alwaysOnTop = IniRead("settings.ini", "settings", "alwaysontop", 0)

$closeMechanism = IniRead("settings.ini", "settings", "closemechanism", "keyup")
$closeMechanism = StringLower($closeMechanism)
If ($closeMechanism <> "keyup" And $closeMechanism <> "keypress" And $closeMechanism <> "closebutton") Then
	MsgBox(0, "warning", "close mechanism not set to a valid option.  Defaulting to 'keyup'")
	$closeMechanism = "keyup"
EndIf

HotKeySet("^"&$hotkey, "LookUpWord")

Opt("TrayMenuMode", 1)

$dictFiles = @ScriptDir & "\"

Dim $gui  ; we have to dim gui as a global variable in case we use <hotkey> to kill it.

$trayVlasisku = TrayCreateItem("Launch vlasisku")
TrayCreateItem("")
$trayConfigure = TrayCreateItem("Configure settings")
$trayRestart = TrayCreateItem("Restart QuickDict")
$trayHelp = TrayCreateItem("Help")
TrayCreateItem("")
$trayExit = TrayCreateItem("Exit")
TraySetState()

While (True)
	Sleep(50)
	$message = TrayGetMsg()
	Select
		Case $message == 0
			ContinueLoop
		Case $message == $trayVlasisku
			ShellExecute("http://vlasisku.lojban.org")
			ContinueLoop
		Case $message == $trayConfigure
			Run("notepad.exe settings.ini")
			Sleep(200)
			MsgBox(0, "Reminder", "Don't forget.  Changes to settings don't take effect until you restart QuickDict")
			ContinueLoop
		Case $message == $trayRestart
			Run("QuickDict.exe")
			Exit
		Case $message == $trayHelp
			If (FileExists(@ScriptDir & "\Readme.txt") == 1) Then
				Run("notepad.exe " & @ScriptDir & "\Readme.txt")
			Else
				MsgBox(0, "Error", "Could not find Readme.txt")
			EndIf
			ContinueLoop
		Case $message == $trayExit
			ExitLoop

		; we can continue adding cases (and corresponding tray items) for other useful functions

	EndSelect
WEnd

Func LookUpWord()

	If (WinExists("QuickDict - Definition") == 1) Then
		; ok, instead of just returning, fine, we'll just kill the old script wherever it was
		killGui()
	EndIf

    $clipBuff = ClipGet()
    Send("^c")
    Sleep(100)
    $word = ClipGet()
	Sleep(100)
	ClipPut($clipBuff)

	If ($word == "" Or $clipBuff == $word) Then
		MsgBox(0, "info", "no text seems to be highlighted")
		Return
	EndIf

    ClipPut($clipBuff)

	$word = StringReplace($word, "", "'")
	$word = StringReplace($word, "`", "'")

	$word = StringStripWS($word, 3)

	$guiHeight = 100
	$guiWidth = 560
	$line = ""
	$ylessWord = StringReplace($word, "y", "")
	If (StringRegExp($ylessWord, "[bcdfgjklmnprstvxz]{2}") == 0) Then
		; we've got a cmavo
		$line = cmavo($word)
		$line = multiLineMe($line, "cmavo")
		$guiHeight = 100  ; cmavo have much shorter definitions, no need to be dynamic to keep up with massive quantities of newlines

	ElseIf (StringLen($word) == 5) Then
		; we've got a gismu
		$line = gismu($word)
		$line = multiLineMe($line, "gismu")
		$guiHeight = 50 + Round(StringLen($line)/125) * 20   ; this tricky bit is to handle absurdly long cf's

	Else
		; we've got a lujvo, a fu'ivla (which I have no idea how to deal with), or nothing
		If ($tryVlasisku == 1) Then
			; ok, grab vlasisku.com/word and see if we get a definition
			$result = askVlasisku($word)
			If ($result == "") Then
				$line = parseLujvo($word)
				$guiWidth = StringLen($line) * 5
				$guiHeight = 30

			Else
				$line = "From: " & parseLujvo($word) & @CRLF & @CRLF
				$line = $line & "vlasisku definition: " & $result
				$guiHeight = 75
			EndIf
		Else
			$line = parseLujvo($word)
			$guiWidth = StringLen($line) * 5
			$guiHeight = 30
		EndIf
	EndIf

	If ($line == "") Then
		; it was nothing that we know about
		Return
	EndIf

	$mouse = MouseGetPos()

	If ($mouse[1] > @DesktopHeight - 100) Then
		$mouse[1] = @desktopHeight - 120
	EndIf

	If ($alwaysOnTop == 0) Then
		$gui = GUICreate("QuickDict - Definition", $guiWidth, $guiHeight, 0, $mouse[1], BitAND($GUI_SS_DEFAULT_GUI, BitNOT($WS_MINIMIZEBOX), BitNOT($WS_MAXIMIZEBOX)))
	Else
		$gui = GUICreate("QuickDict - Definition", $guiWidth, $guiHeight, 0, $mouse[1], BitAND($GUI_SS_DEFAULT_GUI, BitNOT($WS_MINIMIZEBOX), BitNOT($WS_MAXIMIZEBOX)), $WS_EX_TOPMOST)
	EndIf

	GUISetState(@SW_SHOW)
	GUICtrlCreateLabel($line, 10, 10, 550, 200)

	; this option tells the definition window to handle events by calling functions instead of
	; assuming that we will just keep polling the event queue
	Opt("GUIOnEventMode", 1)

	; true "killGui" will also unset the "hotkey" global hotkey.  but who cares
	; about unsetting an already unset hotkey?
	GUISetOnEvent($GUI_EVENT_CLOSE, "killGui")

	; three cases.

	; 1) If we're using keypress to kill the gui, then set the hotkey to killGui() which knows what to do
	If ($closeMechanism == "keypress") Then
		HotKeySet($hotkey, "killGui")
		; ok, killGui will take care of killing the GUI so we're ok to be done
		Return
	EndIf

	; 2) If we're using 'keyup' of the control key, then loop till keyup, then delete gui and return
	If ($closeMechanism == "keyup") Then
		While (True)
			Sleep(100)

			If (_IsPressed('11') == 0 Or _IsPressed('11') == False) Then
				GUIDelete($gui)
				Return
			EndIf
		WEnd
	EndIf

	; 3) if we're just letting the user kill the gui then just... leave it up for the user to close when (s)he see's fit

EndFunc

Func killGui()
	GUIDelete($gui)
	HotKeySet($hotkey) ; unset the hotkey so it's not perpetually trying to kill a non-existent gui
EndFunc

Func askVlasisku($word)
	$tempHtmlFile = @TempDir & "\vlasisku.html"
	FileDelete($tempHtmlFile)

	$vlasiskuUrl = "http://vlasisku.lojban.org/" & $word

	InetGet($vlasiskuUrl, $tempHtmlFile)

	$returnLine = ""

	If (FileExists($tempHtmlFile) == 1) Then
		$vlasiskuFile = FileOpen($tempHtmlFile,0)
		$vlasiskuLine = FileReadLine($vlasiskuFile)
		While (True)
			If (StringInStr($vlasiskuLine, "<p class=""definition"">") > 0) Then
				; we've found our definition line.  Now just gotta parse it.
				; we could probably do this in a much more formal and robust way, but it's late, and I'm tired as fuck-all
				$vlasiskuLine = StringReplace($vlasiskuLine, "<p class=""definition"">", "")
				$vlasiskuLine = StringReplace($vlasiskuLine, "<sub>", "")
				$vlasiskuLine = StringReplace($vlasiskuLine, "</sub>", "")
				$vlasiskuLine = StringReplace($vlasiskuLine, "</p>", "")
				$returnLine = $vlasiskuLine
				ExitLoop

			EndIf
			$vlasiskuLine = FileReadLine($vlasiskuFile)
			If (@error == -1) Then
				ExitLoop
			EndIf
		WEnd
	EndIf
	FileClose($vlasiskuFile)

	FileDelete($tempHtmlFile)

	Return $returnLine
EndFunc

Func multiLineMe($line, $type)
	$returnLine = ""
	If ($type == "gismu") Then
		$returnLine = "gismu:      " & StringMid($line, 2, 5) & @CRLF
		$returnLine = $returnLine & "rafsi:        " & StringMid($line, 7, 13) & @CRLF
		$returnLine = $returnLine & "Definition: " & StringMid($line, 63, 96) & @CRLF

		If (StringLen($line) > 170) Then ; ok, we've got cf then
			$returnLine = $returnLine & "cf:            " & StringMid($line, 170)
		EndIf
		Return $returnLine

	ElseIf ($type == "cmavo") Then
		$returnLine = "cmavo:     " & StringStripWS(StringLeft($line, 11), 3) & @CRLF
		$returnLine = $returnLine & "selma'o:    " & StringStripWS(StringMid($line, 12, 9), 3) & @CRLF
		$returnLine = $returnLine & "gloss:       " & StringStripWS(StringMid($line, 21, 42), 3) & @CRLF
		$returnLine = $returnLine & "definition: " & StringStripWS(StringMid($line, 63, 106), 3)
		If (StringLen($line) > 169) then  ; well, guess we have a cf
			$returnLine =  @CRLF & $returnLine & "cf:         " & StringStripWS(StringMid($line, 169), 3)
		EndIf

	ElseIf ($type == "lujvo") Then
		; no need to do anything fancy.  lujvo (when we're breaking them appart) only ever take up 1 line (unless they're uber-lujvo)
		$returnLine = $line
	Else
		; well I don't know HOW to handle this type, I guess.
		$returnLine = $line
	EndIf

	Return $returnLine

EndFunc

Func parseLujvo($word)

	; what are the different forms of rafsi
	; CVV   = ^[bcdfgjklmnprstvxz]{1}[aeiou]{2}
	; CVC   = ^[bcdfgjklmnprstvxz]{1}[aeiou]{1}[bcdfgjklmnprstvxz]{1}
	; CV'V  = ^[bcdfgjklmnprstvxz]{1}[aeiou]{1}\'[aeiou]{1}
	; CVCCy = ^[bcdfgjklmnprstvxz]{1}[aeiou]{1}[bcdfgjklmnprstvxz]{2}y
	; CCVCy = ^[bcdfgjklmnprstvxz]{2}[aeiou]{1}[bcdfgjklmnprstvxz]{1}y

	$C     = "^[bcdfgjklmnprstvxz]{1}"
	$CVV   = "^[bcdfgjklmnprstvxz]{1}[aeiou]{2}"
	$CCV   = "^[bcdfgjklmnprstvxz]{2}[aeiou]{1}"
	$CVC   = "^[bcdfgjklmnprstvxz]{1}[aeiou]{1}[bcdfgjklmnprstvxz]{1}"
	$CVhV  = "^[bcdfgjklmnprstvxz]{1}[aeiou]{1}\'[aeiou]{1}"
	$CVCCy = "^[bcdfgjklmnprstvxz]{1}[aeiou]{1}[bcdfgjklmnprstvxz]{2}y"
	$CCVCy = "^[bcdfgjklmnprstvxz]{2}[aeiou]{1}[bcdfgjklmnprstvxz]{1}y"
	$CVCCV = "^[bcdfgjklmnprstvxz]{1}[aeiou]{1}[bcdfgjklmnprstvxz]{2}[aeiou]{1}"
	$CCVCV = "^[bcdfgjklmnprstvxz]{2}[aeiou]{1}[bcdfgjklmnprstvxz]{1}[aeiou]{1}"

	; simplify life by checking for invalid endings early on
	If (StringRegExp(StringRight($word, 1), $C) Or StringRight($word, 1) == "y") Then
		Return ""
	EndIf

	$result = ""

	While (StringLen($word) > 2)

		; first, strip any leading "y"s
		If (StringLeft($word, 1) == "y") Then
			$word = StringMid($word, 2)
		EndIf

		; first things first, if we've got only 5 chars left, it could be ending in a gismu
		If (StringLen($word) == 5) Then
			If (StringRegExp($word, $CCVCV) == 1 Or StringRegExp($word, $CVCCV) == 1) Then
				$line = gismu($word)
				If ($line <> "") Then
					$gismu = $word
					$gloss = StringMid($line, 21, 21)
					$gloss = StringStripWS($gloss, 3)
					$result = $result & $gismu & " - " & $gloss & " | "
					$word = StringMid($word, 6)
					ContinueLoop
				Else
					Return ""
				EndIf
			EndIf
		EndIf

		; now take care of the 4 letter rafsi forms (god I hate them)
		If (StringRegExp($word, $CCVCy) == 1 Or StringRegExp($word, $CVCCy) == 1) Then
			$rafs = StringLeft($word, 4)
			$rafsi = $rafs&"a"
			$line = gismu($rafsi)
			If ($line == "") Then
				$rafsi = $rafs&"e"
				$line = gismu($rafsi)
				If ($line == "") Then
					$rafsi = $rafs&"i"
					$line = gismu($rafsi)
					If ($line == "") Then
						$rafsi = $rafs&"o"
						$line = gismu($rafsi)
						If ($line == "") Then
							$rafsi = $rafs&"u"
							$line = gismu($rafsi)
							If ($line == "") Then
								; shit in my mouth
								Return ""
							EndIf
						EndIf
					EndIf
				EndIf
			EndIf
			; by now we damn well better have a line and if we don't then we will have returned
			$gismu = StringMid($line, 1, 6)
			$gloss = StringMid($line, 21, 21)
			$gloss = StringStripWS($gloss, 3)
			$result = $result & $gismu & " - " & $gloss & " | "
			$word = StringMid($word, 6)
			ContinueLoop
		EndIf

		If ( (StringRegExp($word, $CVV) == 1) Or (StringRegExp($word, $CVC) == 1) ) Then
			$line = rafsi(StringLeft($word, 3))
			If ($line == "") Then
				; not a real rafsi and stripping off the leading char would still lead to nothing
				Return ""
			EndIf
			$gismu = StringMid($line, 1, 6)
			$gloss = StringMid($line, 21, 21)
			$gloss = StringStripWS($gloss, 3)
			$result = $result & $gismu & " - " & $gloss & " | "
			$word = StringMid($word, 4)
			ContinueLoop
		EndIf

		If (StringRegExp($word, $CVhV) == 1) Then
			$line = rafsi(StringLeft($word, 4))
			If ($line == "") Then
				; not a real rafsi and stripping off the leading char would still lead to nothing
				Return ""
			EndIf
			$gismu = StringMid($line, 1, 6)
			$gloss = StringMid($line, 21, 21)
			$gloss = StringStripWS($gloss, 3)
			$result = $result & $gismu & " - " & $gloss & " | "
			$word = StringMid($word, 5)
			ContinueLoop
		EndIf

		If (StringRegExp($word, $CCV) == 1) Then
			$line = rafsi(StringLeft($word, 3))
			If ($line == "") Then
				; not a real a real rafsi, but maybe if the first C is an "r" or "n", maybe it's a hyphen
				If (StringLeft($word, 1) == "r" Or StringLeft($word, 1) == "n") Then
					; chop off the potential hyphen and run through loop again
					$word = StringMid($word, 2)
					ContinueLoop
				Else
					Return ""
				EndIf
			EndIf

			$gismu = StringMid($line, 1, 6)
			$gloss = StringMid($line, 21, 21)
			$gloss = StringStripWS($gloss, 3)
			$result = $result & $gismu & " - " & $gloss & " | "
			$word = StringMid($word, 4)
			ContinueLoop
		EndIf

	WEnd

	If ($result <> "") Then
		$result = StringLeft($result, StringLen($result)-3)
	EndIf

	Return $result

EndFunc

Func gismu($word)
	$file = FileOpen($dictFiles&"gismu.txt", 0)

	$line = FileReadLine($file)

	While ($line <> "")
		$contain = StringInStr($line, " " & $word & " ")
		If ($contain > 0 And $contain < 4) Then
			FileClose($file)
			Return $line
		EndIf
		$line = FileReadLine($file)
	WEnd
	FileClose($file)
	Return ""
EndFunc

Func cmavo($word)

	$file = FileOpen($dictFiles&"cmavo.txt", 0)

	$line = FileReadLine($file)

	While ($line <> "")
		; sometimes cmavo are surrounded by spaces
		$contain = StringInStr($line, " "&$word&" ")
		If ($contain > 0 And $contain < 3) Then
			FileClose($file)
			Return $line
		EndIf

		; sometimes they are "." then the cmavo
		$contain = StringInStr($line, "."&$word&" ")
		If ($contain > 0 And $contain < 3) Then
			FileClose($file)
			Return $line
		EndIf
		$line = FileReadLine($file)
	WEnd
	FileClose($file)
	Return ""
EndFunc

Func rafsi($word)
	$file = FileOpen($dictFiles&"gismu.txt", 0)

	$line = FileReadLine($file)

	While ($line <> "")
		$contain = StringInStr($line, " "&$word&" ")
		If ($contain > 5 And $contain < 21) Then
			FileClose($file)
			Return $line
		EndIf
		$line = FileReadLine($file)
	WEnd
	FileClose($file)
	Return ""
EndFunc
